#include <msp.h>
#include "clock.h"

void configureClockSystem(void)
{
    //HFXIN - HFXOUT PJ.2 and PJ.3 pin functions
    PJSEL1 =  0x00;
    PJSEL0 = 0x0C;

    //Waits until power change is done
    while(PCM -> CTL1 & PCM_CTL1_PMR_BUSY);

    //Configure Active Mode 1 to use 48MHz for clock input
    PCM -> CTL0 = PCM_CTL0_KEY_VAL | PCM_CTL0_AMR_1;

    //Waits for transition to complete
    //TODO Manage as a global error logic
    while(PCM->IFG & PCM_IFG_AM_INVALID_TR_IFG)
    {
        PCM->CLRIFG= PCM_CLRIFG_CLR_LPM_INVALID_TR_IFG;
    }

    //Waits until power change is done
    while(PCM -> CTL1 & PCM_CTL1_PMR_BUSY);

    //Configure Flash Wait State to 1 for both banks 0 & 1
    FLCTL->BANK0_RDCTL = (FLCTL->BANK0_RDCTL & ~(FLCTL_BANK0_RDCTL_WAIT_MASK)) |
                FLCTL_BANK0_RDCTL_WAIT_1;
    FLCTL->BANK1_RDCTL = (FLCTL->BANK0_RDCTL & ~(FLCTL_BANK1_RDCTL_WAIT_MASK)) |
                FLCTL_BANK1_RDCTL_WAIT_1 ;

    //Clock System Key Register. This unlocks CS registers
    CS -> KEY = CS_KEY_VAL;

    //DCO Disabled
    CS->CTL0 = 0;

    //MCLK, SMCLK, HSMCLK source = HFXTCLK (48MHz external crystaL)
    //MCLK divider = 1 -> MCLK frequency = 48 MHz
    //HSMCLK divider = 2 -> SMCLK frequency = 24 MHz
    //SMCLK divider = 4 -> SMCLK frequency = 12 MHz
    //ACLK source = LFXTCLK (32768Hz external crystal), divider = 1
    CS->CTL1 = CS_CTL1_DIVS_2 | CS_CTL1_DIVHS_1 | CS_CTL1_SELS__HFXTCLK | CS_CTL1_SELM__HFXTCLK;

    //Use external 48 MHz crystal for HFXT
    CS->CTL2 = CS_CTL2_HFXTFREQ_6 | CS_CTL2_HFXTDRIVE | CS_CTL2_HFXT_EN;

    //Checks oscillator fault flags
    //TODO Manage as a global error logic
    while(CS->IFG& CS_IFG_HFXTIFG)
    {
        CS->CLRIFG= CS_IFG_HFXTIFG;
    }

    //Lock CS Module for unintended access
    CS -> KEY = 0;
}

