#include <msp.h>
#include "timer.h"
#include "common.h"
#include "fifo_function_queue.h"

//Callback function for queue
void (*slow_function_callback)();
void (*fast_function_callback)();

static unsigned int miliseconds_tick = 0;

void configureTimer(void)
{
    //Timer A Configuration - SMCLK / divider of 1 / UP mode
    TIMER_A0 -> CTL |= TIMER_A_CTL_TASSEL_2 + TIMER_A_CTL_ID_0 + TIMER_A_CTL_MC_1;

    //Timer A Control Register - Capture compare interrupts are enabled
    TIMER_A0 -> CCTL[0] |= TIMER_A_CCTLN_CCIE;
    //Timer A Capture/Compare Register 0 - set as 1mS(1/f), being f the clock frequency. No prescaler.
    //TACCR = 1mS / (1 / 12MHz) - 1. 1 is substracted because the flag is triggered one cycle later
    TIMER_A0 -> CCR[0] = 11999;

    NVIC_SetPriority(TA0_0_IRQn,0);
    NVIC_EnableIRQ(TA0_0_IRQn);
}

unsigned int getCurrentTime(void)
{
    return miliseconds_tick;
}

void setSlowFunctionISRCallbackFunction(void *callback_function)
{
    slow_function_callback = callback_function;
}

void setFastFunctionISRCallbackFunction(void *callback_function)
{
    fast_function_callback = callback_function;
}

void TA0_0_IRQHandler(void)
{
    if(TIMER_A0->CCTL[0] & TIMER_A_CCTLN_CCIFG)
    {
        if(miliseconds_tick % FAST_ISR_FUNCTION_CALL_TICK == 0)
        {
            addToFunctionQueue(fast_function_callback);
            __low_power_mode_off_on_exit();
        }

        if(miliseconds_tick == SLOW_ISR_FUNCTION_CALL_TICK)
        {
            miliseconds_tick = 0;
            addToFunctionQueue(slow_function_callback);
            __low_power_mode_off_on_exit();
        }
        miliseconds_tick++;
        TIMER_A0->CCTL[0] &= ~TIMER_A_CCTLN_CCIFG;
    }
}
